@@ -9,8 +9,10 @@
  #include <sys/types.h>
  #include <sys/stat.h>
  #include <fcntl.h>
+ const int O_BINARY = 0;
 #endif
 
+#include <sstream>
 
 #include "lib/base.h"
 #include "lib/formats.h"
@@ -30,37 +32,42 @@ const char TypeErrorMsg[] =
 
 PyObject* py_imread_may_multi(PyObject* self, PyObject* args, bool is_multi) {
     const char* filename;
-    if (!PyArg_ParseTuple(args, "s", &filename)) {
+    const char* formatstr;
+    if (!PyArg_ParseTuple(args, "ss", &filename, &formatstr)) {
         PyErr_SetString(PyExc_RuntimeError,TypeErrorMsg);
         return NULL;
     }
-    const char* formatstr = strrchr(filename, '.');
-    if (!formatstr) {
-        PyErr_SetString(PyExc_RuntimeError, "Cannot read extension");
-        return NULL;
-    }
-    ++formatstr;
 
-    int fd = ::open(filename, O_RDONLY);
+    int fd = ::open(filename, O_RDONLY|O_BINARY);
     if (fd < 0) {
-        PyErr_SetString(PyExc_OSError, "File does not exist");
+        std::stringstream ss;
+        ss << "File `" << filename << "` does not exist";
+        PyErr_SetString(PyExc_OSError, ss.str().c_str());
         return 0;
     }
 
     try {
         std::auto_ptr<ImageFormat> format(get_format(formatstr));
         if (!format.get()) {
-            throw CannotReadError("This format is unknown to imread");
+            std::stringstream ss;
+            ss << "This format (" << formatstr << ") is unknown to imread";
+            throw CannotReadError(ss.str());
         }
         if (is_multi && !format->can_read_multi()) {
-            throw CannotReadError("imread cannot read_multi in this format");
+            std::stringstream ss;
+            ss << "imread cannot read_multi in this format (" << formatstr << ")";
+            if (format->can_read()) {
+                ss << " but read() will work.";
+            }
+            throw CannotReadError(ss.str());
         }
         if (!is_multi && !format->can_read()) {
+            std::stringstream ss;
+            ss << "imread cannot read_in this format (" << formatstr << ")";
             if (format->can_read_multi()) {
-                throw CannotReadError("imread cannot read in this format (but can read_multi!)");
-            } else {
-                throw CannotReadError("imread cannot read in this format");
+                ss << "(but can read_multi!)";
             }
+            throw CannotReadError(ss.str());
         }
 
         NumpyFactory factory;
@@ -99,9 +106,11 @@ PyObject* py_imsave(PyObject* self, PyObject* args) {
         PyErr_SetString(PyExc_RuntimeError,TypeErrorMsg);
         return NULL;
     }
-    int fd = ::open(filename, O_CREAT|O_WRONLY, 0644);
+    int fd = ::open(filename, O_CREAT|O_WRONLY|O_BINARY|O_TRUNC, 0644);
     if (fd < 0) {
-        PyErr_SetString(PyExc_OSError, "File does not exist");
+        std::stringstream ss;
+        ss << "Cannot open file '" << filename << "' for writing";
+        PyErr_SetString(PyExc_OSError, ss.str().c_str());
         return 0;
     }
 
@@ -111,7 +120,9 @@ PyObject* py_imsave(PyObject* self, PyObject* args) {
         std::auto_ptr<byte_sink> output(new fd_source_sink(fd));
         std::auto_ptr<ImageFormat> format(get_format(formatstr));
         if (!format->can_write()) {
-            throw CannotWriteError("Cannot write this format");
+            std::stringstream ss;
+            ss << "Cannot write this format (" << formatstr << ")";
+            throw CannotWriteError(ss.str());
         }
         format->write(&input, output.get());
         Py_RETURN_NONE;
